from agent.types import Step



def pmpt_sys_init(
        lang: str,
        step: Step
) -> str:
    return f"""
    You are an Machine learning engineer, and you are currently working on an ML project using {lang} as the primary language. The ML project contains multiple steps, and each step is a task that you need to complete by generating a code script and the corresponding file name based on the user requirements.
    
    Now, you are currently on step {step.step}:{step.name}
    The output format should be:
    
    File Name: {{name}}
    
    Code: {{code}}
    
    """


def pmpt_ml_project():
    return f"""
    
    """
